from typing import Literal, Optional

from pydantic import BaseModel, EmailStr


class CheckoutModel(BaseModel):
    MerchantOrderNo: str
    Amt: int
    ItemDesc: str = "Online Payment To Neweb pay"
    Email: EmailStr
    CREDIT: Optional[Literal[0, 1]]
    ANDROIDPAY: Optional[Literal[0, 1]]
    SAMSUNGPAY: Optional[Literal[0, 1]]
    LINEPAY: Optional[Literal[0, 1]]
    ImageUrl: Optional[str]
    CreditRed: Optional[Literal[0, 1]]
    CREDITAE: Optional[Literal[0, 1]]
    UNIONPAY: Optional[Literal[0, 1]]
    WEBATM: Optional[Literal[0, 1]]
    VACC: Optional[Literal[0, 1]]
    CVS: Optional[Literal[0, 1]]
    BARCODE: Optional[Literal[0, 1]]
    ALIPAY: Optional[Literal[0, 1]]
    P2G: Optional[Literal[0, 1]]
    CVSCOM: Optional[Literal[0, 1]]

    NotifyURL: Optional[str]
    ReturnURL: Optional[str]
    CustomerURL: Optional[str]

    MerchantID: Optional[str]
    Version: Optional[str] = "1.5"
    RespondType: Optional[str] = "JSON"
    TimeStamp: Optional[str]



import json
import time
from typing import Any, Dict, Tuple

# from fastel.utils import requests
import requests
from requests import Response

from fastel.exceptions import APIException
from fastel.logistics.sf import api_token
from fastel.logistics.sf.cryptor import SFCryptor
from fastel.logistics.sf.models import SFLogisticsModel


class SFLogistics:
    def __init__(
        self,
        merchant_id: str,
        aes_key: str,
        app_key: str,
        secret: str,
        stage: str = "stg",
    ) -> None:
        self.merchant_id = merchant_id
        self.aes_key = aes_key
        self.app_key = app_key
        self.secret = secret
        self.stage = stage
        self.sf_token = api_token(self.logistics_url, self.app_key, self.secret)

    @property
    def cryptor(self) -> SFCryptor:
        return SFCryptor(self.aes_key, self.app_key, self.sf_token)

    @property
    def logistics_url(self) -> str:
        if self.stage in ["stg", "STG"]:
            return "http://api-ifsp.sit.sf.global"
        return "https://api-ifsp.sf.global"

    def generate_headers(self, method: str = "IECS_CREATE_ORDER") -> Dict[str, Any]:
        timestamp = str(int(time.time()))
        headers = {
            "appKey": self.app_key,
            "token": self.sf_token,
            "timestamp": timestamp,
            "nonce": timestamp,
            "msgType": method,
            "Content-Type": "application/json",
        }
        return headers

    def encrypt_data(
        self, data: Dict[str, Any], method: str = "IECS_CREATE_ORDER"
    ) -> Tuple[Dict[str, Any], str]:
        msg = json.dumps(data, ensure_ascii=False)
        headers = self.generate_headers(method)
        print(headers)
        ret_code, encrypt, signature = self.cryptor.encrypt_msg(
            msg=msg, nonce=headers["nonce"], timestamp=headers["timestamp"]
        )
        if ret_code != 0:
            raise APIException(status_code=400, error="SF_Encrypt_Error", detail="")

        headers = {**headers, "signature": signature}
        if encrypt:
            return headers, encrypt
        else:
            return headers, ""

    def create_logistics(self, data: SFLogisticsModel) -> Dict[str, Any]:
        url = self.logistics_url + "/openapi/api/dispatch"

        headers, body = self.encrypt_data(
            data=data.dict(exclude_none=True), method="IUOP_CREATE_ORDER"
        )
        resp: Response = requests.post(url, headers=headers, data=body)
        logistics_resp: Dict[str, Any] = resp.json()
        ret, result = self.cryptor.decrypt_msg(
            post_data=logistics_resp.get("apiResultData", "")
        )
        assert isinstance(result, dict)
        return result

    def query_track(self, data: Dict[str, Any]) -> Dict[str, Any]:
        url = self.logistics_url + "/openapi/api/dispatch"
        headers, body = self.encrypt_data(data=data, method="IUOP_QUERY_TRACK")
        resp: Response = requests.post(url, headers=headers, json=body)
        logistics_resp: Dict[str, Any] = resp.json()

        ret, result = self.cryptor.decrypt_msg(
            post_data=logistics_resp.get("apiResultData", "")
        )
        assert isinstance(result, dict)
        return result



from enum import Enum
from typing import Any, Dict, Literal

from pydantic import BaseModel


class LogisticsStatus(str, Enum):
    error = "error"
    pending = "pending"
    in_delivery = "in_delivery"
    delivered = "delivered"
    exception = "exception"
    center_delivered = "center_delivered"
    store_delivered = "store_delivered"


class LogisticTypes(str, Enum):
    HOME = "HOME"
    CVS = "CVS"
    UNKNOWN = "UNKNOWN"


class LogisticsDataModel(BaseModel):
    provider: str
    order_id: str = ""
    logistics_id: str
    logistics_type: LogisticTypes
    logistics_subtype: str = ""
    logistics_status: LogisticsStatus
    logistics_message: str
    logistics_payment_no: str = ""
    logistics_validate_no: str = ""



from typing import Any, Dict

import requests
from jwt import decode

from fastel.social import SocialModel


class LineLogin:
    def __init__(self, data: SocialModel) -> None:
        self.project_client_id = data.project_client_id
        self.project_api_host = data.project_api_host
        self.client_id = data.client_id
        self.client_secret = data.client_secret

        self.CALLBACK_URL = f"{self.project_api_host}/line/callback"
        self.LINE_URL = "https://access.line.me/oauth2/v2.1/authorize"

    def redirect(self, state: str) -> str:
        url = f"{self.LINE_URL}?response_type=code&client_id={self.client_id}&redirect_uri={self.CALLBACK_URL}&state={state}&scope=profile%20openid%20email"
        return url

    def callback(self, state: Dict[str, Any], code: str) -> Dict[str, Any]:
        print(state)
        resp = requests.post(
            "https://api.line.me/oauth2/v2.1/token",
            data={
                "grant_type": "authorization_code",
                "code": code,
                "client_id": self.client_id,
                "client_secret": self.client_secret,
                "redirect_uri": self.CALLBACK_URL,
            },
        )

        resp_json = resp.json()
        line_token = decode(resp_json["id_token"], options={"verify_signature": False})
        result = {}
        result["social_id"] = line_token.get("sub", "")
        result["provider"] = "line"

        try:
            result["email"] = line_token["email"]
            result["name"] = line_token["name"]
        except Exception:
            print(
                f"[WARNING] fb social user {result['social_id']} doesn't have email or name"
            )

        return result



from typing import Any, Literal, Optional

from pydantic import BaseModel, validator


class EcCheckoutModel(BaseModel):
    MerchantTradeNo: str
    TotalAmount: int
    ItemName: str = "Online Payment To Ecpay"
    TradeDesc: str = "no description"
    ChoosePayment: Literal["ALL", "Credit", "WebATM", "ATM", "CVS", "BARCODE"] = "ALL"
    BindingCard: Literal[0, 1] = 0
    MerchantMemberID: Optional[str]
    RelateNumber: Optional[str]
    PaymentType: str = "aio"
    InvoiceMark: Optional[Literal["Y", "N"]] = "N"
    # only fill this if InvoiceMark is Y
    TaxType: Optional[Literal["1", "2", "3", "9"]]
    ReturnURL: Optional[str]
    OrderResultURL: Optional[str]
    PaymentInfoURL: Optional[str]
    ClientRedirectURL: Optional[str]


class EcGetNoCallback(BaseModel):
    MerchantID: str = ""
    MerchantTradeNo: str = ""
    StoreID: str = ""
    RtnCode: str = ""
    RtnMsg: str = ""
    TradeNo: str = ""
    TradeAmt: int = 0
    PaymentType: str = ""
    TradeDate: Optional[str] = ""
    CheckMacValue: Optional[str] = ""
    BankCode: Optional[str] = ""  # 取號結果ATM
    vAccount: Optional[str] = ""  # 取號結果ATM
    ExpireDate: Optional[str] = ""  # 取號結果ATM、CVS和BARCODE
    PaymentNo: Optional[str] = ""  # 取號結果CVS和BARCODE
    Barcode1: Optional[str] = ""  # 取號結果CVS和BARCODE
    Barcode2: Optional[str] = ""  # 取號結果CVS和BARCODE
    Barcode3: Optional[str] = ""  # 取號結果CVS和BARCODE
    CustomField1: Optional[str] = ""
    CustomField2: Optional[str] = ""
    CustomField3: Optional[str] = ""
    CustomField4: Optional[str] = ""


class EcCallback(BaseModel):
    MerchantID: str
    MerchantTradeNo: str
    RtnCode: str
    RtnMsg: str
    TradeNo: str
    TradeAmt: int
    PaymentDate: Optional[str]
    PaymentType: str
    PaymentTypeChargeFee: Optional[int]
    TradeDate: str
    CheckMacValue: str
    StoreID: Optional[str] = ""
    SimulatePaid: Optional[int] = 0
    CustomField1: Optional[str] = ""
    CustomField2: Optional[str] = ""
    CustomField3: Optional[str] = ""
    CustomField4: Optional[str] = ""

    @validator("RtnMsg")
    def validate_msg(cls, v: str, values: Any, **kwargs: Any) -> str:
        return v.encode("Latin-1").decode()



import time
from datetime import datetime
from typing import Any, Dict

import requests

from fastel.payment.cryptors import EcInvoiceCryptor, SHACryptor
from fastel.payment.ecpay.models.checkout import EcCheckoutModel


class Ecpay:
    def __init__(
        self, merchant_id: str, hash_key: str, hash_iv: str, stage: str = "stg"
    ) -> None:
        self.merchant_id = merchant_id
        self.hash_key = hash_key
        self.hash_iv = hash_iv
        self.stage = stage
        self.cryptor = SHACryptor(hash_key, hash_iv)

    @property
    def payment_host(self) -> str:
        if self.stage in ["prod", "PROD"]:
            return "https://payment.ecpay.com.tw/Cashier/AioCheckOut/V5"
        return "https://payment-stage.ecpay.com.tw/Cashier/AioCheckOut/V5"

    def checkout_request(self, checkout: EcCheckoutModel) -> str:
        payload = checkout.dict()
        now = datetime.now()
        gateway_payload: Dict[str, Any] = {
            **payload,
            "MerchantID": self.merchant_id,
            "PaymentType": "aio",
            "EncryptType": 1,
            "MerchantTradeDate": now.strftime("%Y/%m/%d %H:%M:%S"),
        }
        trade_info = self.cryptor.encrypt(gateway_payload)
        checkout_payload = {**gateway_payload, "CheckMacValue": trade_info}
        row_payload = "\n".join(
            map(
                lambda k: f'<input type="hidden" name="{k[0]}" value="{k[1]}"><br>',
                checkout_payload.items(),
            )
        )
        formdata_payload = f"""
                <html>
                  <body>
                      <form name="ecpay" method="post" action="{self.payment_host}">
                          {row_payload}
                          <input type="hidden" value="Submit"><br>
                      </form>
                      </body>
                      <script type="text/javascript">ecpay.submit();
                  </script>
                </html>
                """

        return formdata_payload


class EcpayInvoice:
    def __init__(
        self, merchant_id: str, hash_key: str, hash_iv: str, stage: str = "stg"
    ) -> None:
        self.merchant_id = merchant_id
        self.hash_key = hash_key
        self.hash_iv = hash_iv
        self.stage = stage
        self.cryptor = EcInvoiceCryptor(hash_key, hash_iv)

    @property
    def invoice_url(self) -> str:
        if self.stage in ["prod", "PROD"]:
            return "https://einvoice.ecpay.com.tw/"
        else:
            return "https://einvoice-stage.ecpay.com.tw/"

    def issue(self, data: Dict[str, Any]) -> Dict[str, Any]:
        encrypted_payload = {
            "MerchantID": self.merchant_id,
            "RqHeader": {
                "Timestamp": str(int(time.time())),
                "Revision": "3.0.0",
            },
            "Data": self.cryptor.encrypt(data),
        }

        try:
            invoice_path = (
                "B2BInvoice/Issue"
                if data.get("CustomerIdentifier", "")
                else "B2CInvoice/Issue"
            )
            resp = requests.post(
                f"{self.invoice_url}/{invoice_path}", json=encrypted_payload
            )
            resp.raise_for_status()
            resp_json = resp.json()
            assert isinstance(resp_json, dict)
            if isinstance(resp_json["Data"], str) and resp_json["TransCode"] == 1:
                result = resp_json
                result["Data"] = self.cryptor.decrypt(resp_json["Data"])
            else:
                result = resp_json
        except requests.HTTPError as exc:
            result = {"error": exc.response.json()}

        return result



from datetime import datetime
from typing import Optional

from pydantic import BaseModel


class GetNumResult(BaseModel):
    MerchantID: str
    Amt: str
    TradeNo: str
    MerchantOrderNo: str
    PaymentType: str
    ExpireDate: Optional[str] = ""
    ExpireTime: Optional[str] = "00:00:00"

    BankCode: Optional[str]
    CodeNo: Optional[str]

    Barcode_1: Optional[str]
    Barcode_2: Optional[str]
    Barcode_3: Optional[str]


class Result(BaseModel):
    Amt: int
    MerchantID: str
    MerchantOrderNo: str
    PaymentType: str
    TradeNo: str
    EscrowBank: Optional[str] = None
    IP: Optional[str] = None
    PayTime: Optional[str] = None
    RespondType: Optional[str] = None
    ExpireDate: Optional[datetime] = None  # 取號回傳
    ExpireTime: Optional[datetime] = None  # 取號回傳
    AuthBank: Optional[str] = None  # 信用卡支付回傳參數
    RespondCode: Optional[str] = None  # 信用卡支付回傳參數
    Auth: Optional[str] = None  # 信用卡支付回傳參數
    Card4No: Optional[str] = None  # 信用卡支付回傳參數
    Card6No: Optional[str] = None  # 信用卡支付回傳參數
    Inst: Optional[int] = None  # 信用卡支付回傳參數
    InstEach: Optional[int] = None  # 信用卡支付回傳參數
    InstFirst: Optional[int] = None  # 信用卡支付回傳參數
    ECI: Optional[str] = None  # 信用卡支付回傳參數
    TokenUseStatus: Optional[str] = None  # 信用卡支付回傳參數
    RedAmt: Optional[int] = None  # 信用卡支付回傳參數
    PaymentMethod: Optional[str] = None  # 信用卡支付回傳參數
    DCC_Amt: Optional[float] = None  # 信用卡支付回傳參數
    DCC_Rate: Optional[float] = None  # 信用卡支付回傳參數
    DCC_Markup: Optional[float] = None  # 信用卡支付回傳參數
    DCC_Currency: Optional[str] = None  # 信用卡支付回傳參數
    DCC_Currency_Code: Optional[int] = None  # 信用卡支付回傳參數
    PayBankCode: Optional[str] = None  # WEBATM、ATM
    PayerAccount5Code: Optional[str] = None  # WEBATM、ATM
    CodeNo: Optional[str] = None  # 超商代碼 & ATM轉帳
    StoreType: Optional[int] = None  # 超商代碼 & 超商物流
    StoreID: Optional[str] = None  # 超商代碼
    Barcode_1: Optional[str] = None  # 超商條碼
    Barcode_2: Optional[str] = None  # 超商條碼
    Barcode_3: Optional[str] = None  # 超商條碼
    PayStore: Optional[str] = None  # 超商條碼
    P2GTradeNo: Optional[str] = None  # ezpay電子錢包
    P2GPaymentType: Optional[str] = None  # ezpay電子錢包
    P2GAmt: Optional[int] = None  # ezpay電子錢包
    StoreCode: Optional[str] = None  # 超商物流
    StoreName: Optional[str] = None  # 超商物流
    StoreAddr: Optional[str] = None  # 超商物流
    TradeType: Optional[int] = None  # 超商物流
    CVSCOMName: Optional[str] = None  # 超商物流
    CVSCOMPhone: Optional[str] = None  # 超商物流
    ChannelID: Optional[str] = None  # 跨境支付
    ChannelNo: Optional[str] = None  # 跨境支付
    BankCode: Optional[str] = None  # ATM轉帳


class CallbackMsg(BaseModel):
    Status: str
    Message: str
    Result: Result


class EncryptedCallback(BaseModel):
    Status: str
    MerchantID: str
    TradeInfo: str


class GetNumMsg(BaseModel):
    Status: str
    Message: str
    Result: GetNumResult



import json
import time

import requests

from fastel.payment.cryptors import AESCryptor
from fastel.payment.neweb.models.callback import (
    CallbackMsg,
    EncryptedCallback,
    GetNumMsg,
)
from fastel.payment.neweb.models.checkout import CheckoutModel
from fastel.payment.neweb.models.invoice import IssueModel, IssueResp


class Newebpay:
    def __init__(
        self, merchant_id: str, hash_key: str, hash_iv: str, stage: str = "stg"
    ) -> None:
        self.merchant_id = merchant_id
        self.hash_key = hash_key
        self.hash_iv = hash_iv
        self.stage = stage
        self.cryptor = AESCryptor(hash_key, hash_iv)

    @property
    def payment_host(self) -> str:
        if self.stage in ["prod", "PROD"]:
            return "https://core.newebpay.com/MPG/mpg_gateway"
        return "https://ccore.newebpay.com/MPG/mpg_gateway"

    def checkout_request(self, checkout: CheckoutModel) -> str:
        checkout.MerchantID = self.merchant_id

        payload = checkout.dict()
        trade_info = self.cryptor.encrypt(payload)

        gateway_payload = {
            "MerchantID": self.merchant_id,
            "TradeInfo": trade_info,
            "TradeSha": self.cryptor.build_trade_sha(trade_info),
            "Version": "1.5",
        }

        row_payload = "\n".join(
            map(
                lambda k: f'<input type="hidden" name="{k[0]}" value="{k[1]}"><br>',
                gateway_payload.items(),
            )
        )
        formdata_payload = f"""
                <html>
                  <body>
                      <form name="neweb" method="post" action="{self.payment_host}">
                          {row_payload}
                          <input type="hidden" value="Submit"><br>
                      </form>
                      </body>
                      <script type="text/javascript">neweb.submit();
                  </script>
                </html>
                """

        return formdata_payload

    def decrypt_callback(self, encrypted_callback: EncryptedCallback) -> CallbackMsg:
        decrypted = self.cryptor.decrypt(encrypted_callback.TradeInfo)
        return CallbackMsg.validate(json.loads(decrypted))

    def decrypt_getnum(self, encrypted_callback: EncryptedCallback) -> GetNumMsg:
        decrypted = self.cryptor.decrypt(encrypted_callback.TradeInfo)
        return GetNumMsg.validate(json.loads(decrypted))


class NewebInvoice:
    def __init__(
        self, merchant_id: str, hash_key: str, hash_iv: str, stage: str = "stg"
    ) -> None:
        self.merchant_id = merchant_id
        self.hash_key = hash_key
        self.hash_iv = hash_iv
        self.stage = stage
        self.cryptor = AESCryptor(hash_key, hash_iv)

    @property
    def invoice_url(self) -> str:
        if self.stage in ["prod", "PROD"]:
            return "https://inv.ezpay.com.tw/Api"
        else:
            return "https://cinv.ezpay.com.tw/Api"

    def issue(self, data: IssueModel) -> IssueResp:
        payload = {
            "RespondType": "JSON",
            "Version": "1.4",
            "TimeStamp": str(int(time.time())),
            **data.dict(exclude_none=True),
        }
        encrypted_payload = {
            "MerchantID_": self.merchant_id,
            "PostData_": self.cryptor.encrypt(payload),
        }

        try:
            resp = requests.post(
                f"{self.invoice_url}/invoice_issue", data=encrypted_payload
            )
            resp.raise_for_status()
            resp_json = resp.json()
            if isinstance(resp_json["Result"], str):
                resp_json["Result"] = json.loads(resp_json["Result"])
            else:
                resp_json["Result"] = None

            resp_model = IssueResp.validate(resp_json)
        except requests.HTTPError as exc:
            err_resp = exc.response.json()
            resp_model = IssueResp.validate(err_resp)

        return resp_model
